/*
 *  $Id: lofs.h,v 1.1 1999/05/10 20:18:29 ezk Exp $
 */

#ifndef __FIST_LOFS_H_
#define __FIST_LOFS_H_

/*
 * STRUCTURES:
 */

/* lofs inode data in memory */
struct lofs_inode_info {
    inode_t *inode;
};

/* lofs super-block data in memory */
struct lofs_sb_info {
    super_block_t *s_sb;
#if 0
    /* not needed b/c linux has an s_root field in struct super_block */
    dentry_t *s_root;
#endif
    // inode_t *s_rootinode;
};

/*
 * MACROS:
 */
// File TO Hidden File
#define ftohf(file) ((file_t *)((file)->private_data))
// Inode TO Private Data
#define itopd(ino) ((struct lofs_inode_info *)(ino)->u.generic_ip)
// Inode TO Hidden Inode
#define itohi(ino) (itopd(ino)->inode)
// Superblock TO Private Data
#define stopd(super) ((struct lofs_sb_info *)(super)->u.generic_sbp)
// Superblock TO Hidden Superblock
#define stohs(super) (stopd(super)->s_sb)
// Dentry TO Hidden Dentry (and private data --- for the future)
#define dtopd(dentry) ((dentry)->d_fsdata)
#define dtohd(dentry) ((dentry_t *)dtopd(dentry))
/* VMArea TO Hidden VMArea */
#define vmatohvma(vma) ((vm_area_t *)((vma)->vm_private_data))

#define sbt(sb) ((sb)->s_type->name)

/*
 * EXTERNALS:
 */
extern struct file_operations lofs_fops;
extern struct inode_operations lofs_iops;
extern struct super_operations lofs_sops;
extern struct dentry_operations lofs_dops;
extern struct vm_operations_struct lofs_shared_vmops;
extern struct vm_operations_struct lofs_private_vmops;

extern int lofs_decode_block(const char *inbuf, char *outbuf, int len);
extern int lofs_encode_block(const char *inbuf, char *outbuf, int len);
extern int lofs_interpose(dentry_t *hidden_dentry, dentry_t *this_dentry, super_block_t *sb, int flag);

#ifdef FIST_DEBUG
# define lofs_hidden_dentry(d) __lofs_hidden_dentry(__FILE__,__FUNCTION__,__LINE__,(d))
extern dentry_t *__lofs_hidden_dentry(char *file, char *func, int line, dentry_t *this_dentry);
#else /* not FIST_DEBUG */
# define lofs_hidden_dentry(dentry) dtohd(dentry)
#endif /* not FIST_DEBUG */

#endif	/* __FIST_LOFS_H_ */

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
